# POPFILE LOADABLE MODULE
package Proxy::MERC;

use Proxy::Proxy;
@ISA = ("Proxy::Proxy");

# ---------------------------------------------------------------------------------------------
#
# This module handles communications with Mercury/32 for POPFile.
my $version =  "1.2.2";
#
# Copyright (c) 2001-2003 John Graham-Cumming
#
#   This file is part of POPFile
#
#   POPFile is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   POPFile is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with POPFile; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#   Modified by     Eduardo Subelman(subelman@markmatrix.com)
#
# ---------------------------------------------------------------------------------------------

use strict;
use warnings;
use locale;

# A handy variable containing the value of an EOL for networks
my $eol = "\015\012";

#----------------------------------------------------------------------------
# new
#
#   Class new() function
#----------------------------------------------------------------------------
sub new
{
    my $type = shift;
    my $self = Proxy::Proxy->new();

    # Must call bless before attempting to call any methods

    bless $self, $type;

    $self->name( 'merc' );

    $self->{child_} = \&child__;
    $self->{connection_timeout_error_} = '-ERR';
    $self->{connection_failed_error_}  = '-ERR could not open file';
    $self->{good_response_}            = '^\+OK';

    return $self;
}

# ---------------------------------------------------------------------------------------------
#
# initialize
#
# Called to initialize the MERC proxy module
#
# ---------------------------------------------------------------------------------------------
sub initialize
{
    my ( $self ) = @_;

    # Enabled by default
    $self->config_( 'enabled', 1);

    # By default we don't fork on Windows
    $self->config_( 'force_fork', ($^O eq 'MSWin32')?0:1 );

    # Default ports for service and the user interface
    # use sunrpc port because it is not used in Windows(?)
    $self->config_( 'port', 111 );

    # Only accept connections from the local machine
    $self->config_( 'local', 1 );

    $self->log_( 'Initialized merc ' . $version );

    return $self->SUPER::initialize();;
}

# ---------------------------------------------------------------------------------------------
#
# start
#
# ---------------------------------------------------------------------------------------------
sub start
{
    my ( $self ) = @_;

    # If we are not enabled then no further work happens in this module

    if ( $self->config_( 'enabled' ) == 0 ) {
        return 2;
    }

    # Tell the user interface module that we having a configuration
    # item that needs a UI component

    $self->register_configuration_item_( 'configuration',              # PROFILE BLOCK START
                                         'merc_port',
                                         $self );                      # PROFILE BLOCK STOP

    $self->register_configuration_item_( 'configuration',              # PROFILE BLOCK START
                                         'merc_force_fork',
                                         $self );                      # PROFILE BLOCK STOP

    $self->register_configuration_item_( 'security',                   # PROFILE BLOCK START
                                         'merc_local',
                                         $self );                      # PROFILE BLOCK STOP

    $self->log_( 'Started merc ' . $version );

    return $self->SUPER::start();
}

# ---------------------------------------------------------------------------------------------
#
# child__
#
# The worker method that is called when we get a good connection from a client
#
# $client         - an open stream to a MERC client
# $download_count - The unique download count for this session
# $pipe           - The pipe to the parent process to send messages to
# $ppipe          - 0 or the parent's end of the pipe
# $pid            - 0 if this is a child process
# $session        - API session key
#
# ---------------------------------------------------------------------------------------------
sub child__
{
    my ( $self, $client, $download_count, $pipe, $ppipe, $pid, $session ) = @_;

    # Hash of indexes of downloaded messages

    my %downloaded;

    # Tell the client that we are ready for commands and identify our version number

    $self->tee_( $client, "+OK " . "$eol" );

    # Retrieve commands from the client and process them until the client disconnects or
    # we get a specific QUIT command

    my $count = 0;
    while  ( <$client> ) {
        my $inputfilename;

        $inputfilename = $_;

        # Clean up the inputfilename so that it has a nice clean $eol at the end

        $inputfilename =~ s/(\015|\012)//g;

        $self->log_( "Command: --$inputfilename--" );

        # we now have a file name
        if (open(RETRFILE, "<$inputfilename")) {
            my $class;

            $count++;
            my $history_file;
            ( $class, $history_file ) = $self->{classifier__}->classify_and_modify( $session,  \*RETRFILE, \*NULLFILE, $download_count, $count, 0, '', 0 );

            # Tell the parent that we just handled a mail

            print $pipe "CLASS:$class $session$eol";
            print $pipe "NEWFL:$history_file$eol";
            flush $pipe;
            $self->yield_( $ppipe, $pid );
            close RETRFILE;
            # Add the XTC header
            my $xtc = '';
            if ($self->{classifier__}->get_bucket_parameter( $session, $class, 'xtc' )) {
                $xtc = "$class";
            }
            # Add the Subject modifier
            my $sub = '';
            if ($self->{classifier__}->get_bucket_parameter( $session, $class, 'subject' )) {
                $sub = $self->{classifier__}->config_('subject_mod_left') . "$class" .
                      $self->{classifier__}->config_('subject_mod_right');
            }
            # Add the XPL header
            my $xpl = '';
            if ($self->{classifier__}->get_bucket_parameter( $session, $class, 'xpl' )) {
                $xpl .= "http://";
                $xpl .= $self->module_config_( 'html', 'local' )?"127.0.0.1":$self->module_config_( 'bayes', 'hostname' );
                $xpl .= ":" . $self->module_config_( 'html', 'port' ) . "/jump_to_message?view=$history_file";
            }

            # Tell the client about it
            $self->tee_( $client, "+OK " . "$xtc" . "+" . "$sub" . "+" . $xpl . "$eol" );

            close $client;
            print $pipe "CMPLT$eol";
            flush $pipe;
            $self->yield_( $ppipe, $pid );
            close $pipe;

            $self->log_( "MERC forked child done" );
            last;
        }
        else {
            $self->tee_( $client, "-ERR " . "$eol" );
        }
    }
}

# ---------------------------------------------------------------------------------------------
#
# configure_item
#
#    $name            The name of the item being configured, was passed in by the call
#                     to register_configuration_item
#    $language        Reference to the hash holding the current language
#    $session_key     The current session key
#
#  Must return the HTML for this item
# ---------------------------------------------------------------------------------------------

sub configure_item
{
    my ( $self, $name, $language, $session_key ) = @_;

    my $body;

    # MERC Listen Port widget
    if ( $name eq 'merc_port' ) {
        $body .= "<form action=\"/configuration\">\n";
        $body .= "<label class=\"configurationLabel\" for=\"configMercPort\">$$language{Configuration_MERCPort}:</label><br />\n";
        $body .= "<input name=\"merc_port\" type=\"text\" id=\"configMercPort\" value=\"" . $self->config_( 'port' ) . "\" />\n";
        $body .= "<input type=\"submit\" class=\"submit\" name=\"update_merc_port\" value=\"$$language{Apply}\" />\n";
        $body .= "<input type=\"hidden\" name=\"session\" value=\"$session_key\" />\n</form>\n";
    }

    # Accept MERC from Remote Machines widget
    if ( $name eq 'merc_local' ) {
        $body .= "<span class=\"securityLabel\">$$language{Security_MERC}:</span><br />\n";

        $body .= "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" summary=\"\"><tr><td nowrap=\"nowrap\">\n";
        if ( $self->config_( 'local' ) == 1 ) {
            $body .= "<form class=\"securitySwitch\" action=\"/security\">\n";
            $body .= "<span class=\"securityWidgetStateOff\">$$language{Security_NoStealthMode}</span>\n";
            $body .= "<input type=\"submit\" class=\"toggleOn\" id=\"securityAcceptMERCOn\" name=\"toggle\" value=\"$$language{ChangeToYes}\" />\n";
            $body .= "<input type=\"hidden\" name=\"merc_local\" value=\"1\" />\n";
            $body .= "<input type=\"hidden\" name=\"session\" value=\"$session_key\" />\n</form>\n";
        } else {
            $body .= "<form class=\"securitySwitch\" action=\"/security\">\n";
            $body .= "<span class=\"securityWidgetStateOn\">$$language{Yes}</span>\n";
            $body .= "<input type=\"submit\" class=\"toggleOff\" id=\"securityAcceptMERCOff\" name=\"toggle\" value=\"$$language{ChangeToNo} (Stealth Mode)\" />\n";
            $body .= "<input type=\"hidden\" name=\"merc_local\" value=\"2\" />\n";
            $body .= "<input type=\"hidden\" name=\"session\" value=\"$session_key\" />\n</form>\n";
        }
        $body .= "</td></tr></table>\n";
     }

    if ( $name eq 'merc_force_fork' ) {
        $body .= "<span class=\"configurationLabel\">$$language{Configuration_MERCFork}:</span><br />\n";
        $body .= "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" summary=\"\"><tr><td nowrap=\"nowrap\">\n";

        if ( $self->config_( 'force_fork' ) == 0 ) {
            $body .= "<form action=\"/configuration\">\n";
            $body .= "<span class=\"securityWidgetStateOff\">$$language{No}</span>\n";
            $body .= "<input type=\"submit\" class=\"toggleOn\" id=\"windowTrayIconOn\" name=\"toggle\" value=\"$$language{ChangeToYes}\" />\n";
            $body .= "<input type=\"hidden\" name=\"merc_force_fork\" value=\"1\" />\n";
            $body .= "<input type=\"hidden\" name=\"session\" value=\"$session_key\" />\n</form>\n";
        } else {
            $body .= "<form action=\"/configuration\">\n";
            $body .= "<span class=\"securityWidgetStateOn\">$$language{Yes}</span>\n";
            $body .= "<input type=\"submit\" class=\"toggleOn\" id=\"windowTrayIconOff\" name=\"toggle\" value=\"$$language{ChangeToNo}\" />\n";
            $body .= "<input type=\"hidden\" name=\"merc_force_fork\" value=\"0\" />\n";
            $body .= "<input type=\"hidden\" name=\"session\" value=\"$session_key\" />\n</form>\n";
        }
        $body .= "</td></tr></table>\n";
    }

    return $body;
}

# ---------------------------------------------------------------------------------------------
#
# validate_item
#
#    $name            The name of the item being configured, was passed in by the call
#                     to register_configuration_item
#    $language        Reference to the hash holding the current language
#    $form            Hash containing all form items
#
#  Must return the HTML for this item
# ---------------------------------------------------------------------------------------------

sub validate_item
{
    my ( $self, $name, $language, $form ) = @_;

    if ( $name eq 'merc_port' ) {
        if ( defined($$form{merc_port}) ) {
            if ( ( $$form{merc_port} >= 1 ) && ( $$form{merc_port} < 65536 ) ) {
                $self->config_( 'port', $$form{merc_port} );
                return '<blockquote>' . sprintf( $$language{Configuration_MERCUpdate} . '</blockquote>' , $self->config_( 'port' ) );
             } else {
                 return "<blockquote><div class=\"error01\">$$language{Configuration_Error3}</div></blockquote>";
             }
        }
    }

    if ( $name eq 'merc_local' ) {
        $self->config_( 'local', $$form{merc_local}-1 ) if ( defined($$form{merc_local}) );
    }

    if ( $name eq 'merc_force_fork' ) {
        if ( defined($$form{merc_force_fork}) ) {
            $self->config_( 'force_fork', $$form{merc_force_fork} );
        }
    }

    return '';
}

# required et al.
